import { useGameProps } from "./useGameProps"
import { socket } from "@lib/socket"
import status from "http-status"
import { useSession } from "next-auth/react"
import { useRouter } from "next/router"
import { useEffect, useMemo, useState } from "react"
import { toast } from "react-toastify"

/** This function should only be called once per page, otherwise there will be multiple socket connections and duplicate event listeners. */
function useSocket() {
  const [isConnectedState, setIsConnectedState] = useState(false)
  const {
    payload,
    userStates,
    setPlayer,
    setSetting,
    full,
    setIsReady,
    setIsConnected,
    hash: stateHash,
  } = useGameProps()
  const { data: session } = useSession()
  const router = useRouter()

  const { i, isIndex } = useMemo(() => {
    const i = payload?.users.findIndex((user) => session?.user?.id === user?.id)
    const isIndex = !(i === undefined || i < 0)
    if (!isIndex) return { i: undefined, isIndex }
    return { i, isIndex }
  }, [payload?.users, session?.user?.id])

  useEffect(() => {
    if (!isIndex) return
    setIsConnected({
      i,
      isConnected: isConnectedState,
    })
  }, [i, isConnectedState, isIndex, setIsConnected])

  useEffect(() => {
    if (!session?.user.id) return
    socket.connect()

    socket.on("connect", () => {
      console.log("connected")
      toast.dismiss("connect_error")
      setIsConnectedState(true)
    })

    socket.on("connect_error", (error) => {
      console.log("Connection error:", error.message)
      if (error.message === status["403"]) router.push("/")
      if (error.message !== "xhr poll error") return
      const toastId = "connect_error"
      const isActive = toast.isActive(toastId)
      console.log(toastId, isActive)
      if (isActive)
        toast.update(toastId, {
          autoClose: 5000,
        })
      else
        toast.warn("Es gibt Probleme mit der Echtzeitverbindung.", { toastId })
    })

    socket.on("gameSetting", (payload, hash) => {
      const newHash = setSetting(payload)
      if (!newHash || newHash === hash) return
      console.log("hash", hash, newHash)
      socket.emit("update", (body) => {
        console.log("update")
        full(body)
      })
    })

    socket.on("playerEvent", (event) => {
      const { type, i } = event
      let message: string
      console.log(type)
      switch (type) {
        case "disconnect":
          setIsConnected({
            i,
            isConnected: false,
          })
          message = "Player is disconnected."
          break

        case "leave":
          message = "Player has left the lobby."
          break

        case "connect":
          setIsConnected({
            i,
            isConnected: true,
          })
          socket.emit("isReady", userStates[i].isReady)
          message = "Player has joined the lobby."
          break

        default:
          message = "Not defined yet."
          break
      }
      toast.info(message, { toastId: message })
      if (type === "disconnect") return
      const { payload, hash } = event
      const newHash = setPlayer(payload)
      console.log(newHash, hash, !newHash, newHash === hash)
      if (!newHash || newHash === hash) return
      console.log("hash", hash, newHash)
      socket.emit("update", (body) => {
        console.log("update")
        full(body)
      })
    })

    socket.on("isReady", (payload) => {
      setIsReady(payload)
    })

    socket.on("disconnect", () => {
      console.log("disconnect")
      setIsConnectedState(false)
    })

    return () => {
      socket.removeAllListeners()
    }
  }, [
    full,
    router,
    session?.user.id,
    setIsConnected,
    setIsReady,
    setPlayer,
    setSetting,
    userStates,
  ])

  // useEffect(() => {
  //   if (!isConnected) return
  //   let count = 0
  //   const interval = setInterval(() => {
  //     const start = Date.now()
  //     socket.volatile.emit("ping", ++count, (count) => {
  //       const duration = Date.now() - start
  //       console.log("ping", count, duration)
  //     })
  //   }, 5000)
  //   return () => clearInterval(interval)
  // }, [isConnected])

  return { isConnected: isIndex ? userStates[i].isConnected : isConnectedState }
}

export default useSocket
